/*
 * arch/arm/mach-versatile/include/mach/ipc.h
 *
 * Copyright (c) ARM Limited 2003.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _BCM2708_IPC_H_
#define _BCM2708_IPC_H_

#include <asm/io.h>

#define ENABLE_L2_CACHE

#define IPC_BLOCK_MAGIC			0xCAFEBABE

#define IPC_BLOCK_MAGIC_OFFSET		0x000

#define IPC_BLOCK_INFO_OFFSET		0x200

#define IPC_VC_ARM_INTERRUPT_OFFSET	0x800  /* VC  -> ARM */
#define IPC_ARM_VC_INTERRUPT_OFFSET	0x804  /* ARM -> VC */

//currently we tie the interrupt number to the user slot
#define IPC_BLOCK_MAX_NUM_USERS		32

#define IPC_BLOCK_SIZE			SZ_2M

#define IPC_IRQNUM_NONE		-1

#define ARM_BASE_ADDR_IN_VC_MAP_128	(0x03000000)   /* offset of ARM 0x0 address in VC memory map for 128MB SDRAM*/
#define ARM_BASE_ADDR_IN_VC_MAP_256	(0x04800000)   /* offset of ARM 0x0 address in VC memory map for 256MB SDRAM*/
#ifdef ENABLE_L2_CACHE
#define _REAL_BUS_OFFSET                (0x40000000)   /* enable L2 cache */
#else
#define _REAL_BUS_OFFSET			(0xC0000000)   /* don't use L1 or L2 caches */
#endif
#define VC_BASE_ADDR_IN_ARM_MAP		(0x0B800000) /* offset to the VC physical 0x0 in ARM's view */

extern uint32_t dynamic_arm_base_addr;
extern uint32_t vc_sdram_mb_size;
extern uint32_t vc_factory_reset;

extern int ipc_init(void);
extern int ipc_notify_vc_event(uint32_t ipc_id);
extern int ipc_lookup_irqnum(u32 four_cc);
extern void *ipc_bus_to_virt(uint32_t bus_addr);

typedef struct uboot_args{
    uint32_t    sdram_size;
}uboot_args_t;

static inline uint32_t __ipc_bus_to_phys(const uint32_t vc_phy_addr)
{
	if( (vc_phy_addr - _REAL_BUS_OFFSET) < dynamic_arm_base_addr)
		return( vc_phy_addr + VC_BASE_ADDR_IN_ARM_MAP);
	else{
		return( vc_phy_addr - dynamic_arm_base_addr);
	}
}

static inline uint32_t __arm_to_vc_phys(const uint32_t arm_phy_addr)
{
	if(arm_phy_addr >= VC_BASE_ADDR_IN_ARM_MAP)
		return( arm_phy_addr + _REAL_BUS_OFFSET - VC_BASE_ADDR_IN_ARM_MAP);
	else
		return( arm_phy_addr + _REAL_BUS_OFFSET + dynamic_arm_base_addr);
}

static inline uint32_t __ipc_phys_to_bus(const uint32_t arm_phy_addr)
{
    return (__arm_to_vc_phys(arm_phy_addr));
}

static inline void __init_ipc_addr_translation()
{
    uboot_args_t * magic_args = (uboot_args_t *)(VC_BASE_ADDR_IN_ARM_MAP + ARM_BASE_ADDR_IN_VC_MAP_128 - (1024 * sizeof(uint32_t)));

    vc_sdram_mb_size = magic_args->sdram_size & 0x7fffffff;
    vc_factory_reset = magic_args->sdram_size & 0x80000000;
    dynamic_arm_base_addr = vc_sdram_mb_size > 128 ? ARM_BASE_ADDR_IN_VC_MAP_256 : ARM_BASE_ADDR_IN_VC_MAP_128;
}

#if 0
static inline int ipc_regs_wait_for_register( uint32_t reg, uint32_t bitmask,
        uint32_t required, uint32_t timeout, uint32_t cond)
{
    uint32_t val;
    uint32_t cnt = timeout;

    printf("wait for reg %x %x\n", reg, bitmask);

    while (1) {
        val = readl(reg);
        printf("got val %x\n", val);
        if ((bitmask & val)) {
            if (required && (bitmask & val) != required) {
                continue;
            }

            if (val & ~bitmask) {
                /* Default vc implementation has very strict rules about conditions
                 * which don't seem to be needed. If you get this error and ipc isn't working,
                 * this is a possibile error source. */
                printf("Warning: possible error, using out of spec ipc reg wait.\n");
            }
            break;
        }
        if (timeout != -1) {
            if (!cnt)
                return -1;
            cnt--;
            udelay(1000);
        }
    }

    return 0;
}
#endif

#endif  /* _BCM2708_IPC_H_ */
