/*=============================================================================
Copyright (c) 2007 Broadcom Europe Limited.
All rights reserved.

Project  :  vcfw
Module   :  ipc driver
File     :  $RCSfile: adc.h,v $
Revision :  $Revision: #1 $

FILE DESCRIPTION

=============================================================================*/

#ifndef MEDIA_DEC_REGS_H_
#define MEDIA_DEC_REGS_H_


#define MEDIA_DEC_NEXT(last_, last_bytes_, alignment_) ((last_ + last_bytes_ + alignment_ - 1) & ~(alignment_ - 1))

typedef struct
{
   uint32_t buffer_id;
   uint32_t buffer_size;
   uint32_t buffer_filled_size;
   uint32_t buffer_ptr;
   int64_t  timestamp;
   uint32_t sequence_number;
   uint32_t flags;
   uint32_t callback_context;
   uint32_t _pad;
} MEDIA_DEC_FIFO_ENTRY_T;


#define MEDIA_DEC_CONTROL_OFFSET                    0x00
   #define MEDIA_DEC_CONTROL_ENABLE_BIT             0x00000001
   #define MEDIA_DEC_CONTROL_PLAY_BIT               0x00000002

   // The following are independent bits of the above control bits;
   // set them to indicate you want the following requests
   #define MEDIA_DEC_CONTROL_SET_FLAGS_BIT          0x00000004
   #define MEDIA_DEC_CONTROL_SET_SRC_REGION_BIT     0x00000008
   #define MEDIA_DEC_CONTROL_SET_TARGET_REGION_BIT  0x00000010
   #define MEDIA_DEC_CONTROL_SET_VOLUME_BIT         0x00000020
   #define MEDIA_DEC_CONTROL_GET_VOLUME_BIT         0x00000040
   #define MEDIA_DEC_CONTROL_GET_VIDEO_LEVEL_BIT    0x00000080
   #define MEDIA_DEC_CONTROL_GET_AUDIO_LEVEL_BIT    0x00000100
   #define MEDIA_DEC_CONTROL_SET_MUTED_BIT          0x00000200
   #define MEDIA_DEC_CONTROL_SET_TRANSPARENCY_BIT   0x00000400
   #define MEDIA_DEC_CONTROL_GET_FRAME_BIT          0x00000800

   #define MEDIA_DEC_CONTROL_LOCAL_DATAMODE_BIT     0x20000000
   #define MEDIA_DEC_CONTROL_LOCAL_FILEMODE_BIT     0x40000000
   #define MEDIA_DEC_CONTROL_ERROR_BIT              0x80000000
#define MEDIA_DEC_STATUS_OFFSET                     MEDIA_DEC_NEXT(MEDIA_DEC_CONTROL_OFFSET, 4, 4)

#define MEDIA_DEC_SOURCE_X_OFFSET                   MEDIA_DEC_NEXT(MEDIA_DEC_STATUS_OFFSET, 4, 16)
#define MEDIA_DEC_SOURCE_Y_OFFSET                   MEDIA_DEC_NEXT(MEDIA_DEC_SOURCE_X_OFFSET, 4, 4)
#define MEDIA_DEC_SOURCE_WIDTH_OFFSET               MEDIA_DEC_NEXT(MEDIA_DEC_SOURCE_Y_OFFSET, 4, 4)
#define MEDIA_DEC_SOURCE_HEIGHT_OFFSET              MEDIA_DEC_NEXT(MEDIA_DEC_SOURCE_WIDTH_OFFSET, 4, 4)

#define MEDIA_DEC_TARGET_DISPLAY_OFFSET             MEDIA_DEC_NEXT(MEDIA_DEC_SOURCE_HEIGHT_OFFSET, 4, 16)
#define MEDIA_DEC_TARGET_LAYER_OFFSET               MEDIA_DEC_NEXT(MEDIA_DEC_TARGET_DISPLAY_OFFSET, 4, 4)
#define MEDIA_DEC_TARGET_X_OFFSET                   MEDIA_DEC_NEXT(MEDIA_DEC_TARGET_LAYER_OFFSET, 4, 4)
#define MEDIA_DEC_TARGET_Y_OFFSET                   MEDIA_DEC_NEXT(MEDIA_DEC_TARGET_X_OFFSET, 4, 4)
#define MEDIA_DEC_TARGET_WIDTH_OFFSET               MEDIA_DEC_NEXT(MEDIA_DEC_TARGET_Y_OFFSET, 4, 4)
#define MEDIA_DEC_TARGET_HEIGHT_OFFSET              MEDIA_DEC_NEXT(MEDIA_DEC_TARGET_WIDTH_OFFSET, 4, 4)
#define MEDIA_DEC_TARGET_ASPECT_MODE                MEDIA_DEC_NEXT(MEDIA_DEC_TARGET_HEIGHT_OFFSET, 4, 4)
#define MEDIA_DEC_TARGET_PIXEL_ASPECT               MEDIA_DEC_NEXT(MEDIA_DEC_TARGET_ASPECT_MODE, 4, 4)

#define MEDIA_DEC_VIDEO_TYPE                        MEDIA_DEC_NEXT(MEDIA_DEC_TARGET_PIXEL_ASPECT, 4, 16)
#define MEDIA_DEC_AUDIO_TYPE                        MEDIA_DEC_NEXT(MEDIA_DEC_VIDEO_TYPE, 4, 4)
#define MEDIA_DEC_RESOLUTION_X                      MEDIA_DEC_NEXT(MEDIA_DEC_AUDIO_TYPE, 4, 4)
#define MEDIA_DEC_RESOLUTION_Y                      MEDIA_DEC_NEXT(MEDIA_DEC_RESOLUTION_X, 4, 4)
#define MEDIA_DEC_FRAMES                            MEDIA_DEC_NEXT(MEDIA_DEC_RESOLUTION_Y, 4, 4)

#define MEDIA_DEC_PLAYBACK_TIME                     MEDIA_DEC_NEXT(MEDIA_DEC_FRAMES, 4, 16)

#define MEDIA_DEC_DEBUG_MASK                        MEDIA_DEC_NEXT(MEDIA_DEC_PLAYBACK_TIME, 4, 4)

#define MEDIA_DEC_DEBUG_FILENAME                    MEDIA_DEC_NEXT(MEDIA_DEC_DEBUG_MASK, 4, 32)
#define MEDIA_DEC_DEBUG_FILENAME_LENGTH             256

#define MEDIA_DEC_AUDIO0_DATA_ADDRESS_OFFSET        MEDIA_DEC_NEXT(MEDIA_DEC_DEBUG_FILENAME, MEDIA_DEC_DEBUG_FILENAME_LENGTH, 16)
#define MEDIA_DEC_AUDIO0_DATA_LENGTH_OFFSET         MEDIA_DEC_NEXT(MEDIA_DEC_AUDIO0_DATA_ADDRESS_OFFSET, 4, 4)
#define MEDIA_DEC_AUDIO1_DATA_ADDRESS_OFFSET        MEDIA_DEC_NEXT(MEDIA_DEC_AUDIO0_DATA_LENGTH_OFFSET, MEDIA_DEC_DEBUG_FILENAME_LENGTH, 16)
#define MEDIA_DEC_AUDIO1_DATA_LENGTH_OFFSET         MEDIA_DEC_NEXT(MEDIA_DEC_AUDIO1_DATA_ADDRESS_OFFSET, 4, 4)

#define MEDIA_DEC_VIDEO0_DATA_ADDRESS_OFFSET         MEDIA_DEC_NEXT(MEDIA_DEC_AUDIO1_DATA_LENGTH_OFFSET, 4, 16)
#define MEDIA_DEC_VIDEO0_DATA_LENGTH_OFFSET          MEDIA_DEC_NEXT(MEDIA_DEC_VIDEO0_DATA_ADDRESS_OFFSET, 4, 4)
#define MEDIA_DEC_VIDEO1_DATA_ADDRESS_OFFSET         MEDIA_DEC_NEXT(MEDIA_DEC_VIDEO0_DATA_LENGTH_OFFSET, 4, 4)
#define MEDIA_DEC_VIDEO1_DATA_LENGTH_OFFSET          MEDIA_DEC_NEXT(MEDIA_DEC_VIDEO1_DATA_ADDRESS_OFFSET, 4, 4)

#define MEDIA_DEC_CAPTURE_FRAGMENT_LIST_ADDRESS     MEDIA_DEC_NEXT(MEDIA_DEC_VIDEO1_DATA_LENGTH_OFFSET, 4, 16)
#define MEDIA_DEC_CAPTURE_FRAGMENT_LIST_ENTRIES     MEDIA_DEC_NEXT(MEDIA_DEC_CAPTURE_FRAGMENT_LIST_ADDRESS, 4, 4)
#define MEDIA_DEC_CAPTURE_WIDTH                     MEDIA_DEC_NEXT(MEDIA_DEC_CAPTURE_FRAGMENT_LIST_ENTRIES, 4, 4)
#define MEDIA_DEC_CAPTURE_HEIGHT                    MEDIA_DEC_NEXT(MEDIA_DEC_CAPTURE_WIDTH, 4, 4)
#define MEDIA_DEC_CAPTURE_STRIDE                    MEDIA_DEC_NEXT(MEDIA_DEC_CAPTURE_HEIGHT, 4, 4)

#define MEDIA_DEC_REGS_VIDEO_FIFO_ENTRIES (32 + 1) // Can never be full, hence the +1
#define MEDIA_DEC_REGS_AUDIO_FIFO_ENTRIES (32 + 1) // Can never be full, hence the +1

#define MEDIA_DEC_REGS_VIDEO_OUT_FIFO_REGS_BASE          MEDIA_DEC_NEXT(MEDIA_DEC_CAPTURE_STRIDE, 4, 32)
#define MEDIA_DEC_VIDEO_OUT_WRITE_PTR_OFFSET             (MEDIA_DEC_REGS_VIDEO_OUT_FIFO_REGS_BASE + 0x00)
#define MEDIA_DEC_VIDEO_OUT_READ_PTR_OFFSET              (MEDIA_DEC_REGS_VIDEO_OUT_FIFO_REGS_BASE + 0x04)
#define MEDIA_DEC_VIDEO_OUT_FIFO_SIZE_OFFSET             (MEDIA_DEC_REGS_VIDEO_OUT_FIFO_REGS_BASE + 0x08)
#define MEDIA_DEC_VIDEO_OUT_FIFO_ENTRY_OFFSET            (MEDIA_DEC_REGS_VIDEO_OUT_FIFO_REGS_BASE + 0x0c)
#define MEDIA_DEC_VIDEO_OUT_FIFO_START_OFFSET            (MEDIA_DEC_REGS_VIDEO_OUT_FIFO_REGS_BASE + 0x10)
#define MEDIA_DEC_VIDEO_OUT_FIFO_OFFSET                  (MEDIA_DEC_REGS_VIDEO_OUT_FIFO_REGS_BASE + 0x20)

#define MEDIA_DEC_REGS_VIDEO_IN_FIFO_REGS_BASE          MEDIA_DEC_NEXT(MEDIA_DEC_VIDEO_OUT_FIFO_OFFSET, MEDIA_DEC_REGS_VIDEO_FIFO_ENTRIES * sizeof(MEDIA_DEC_FIFO_ENTRY_T), 32)
#define MEDIA_DEC_VIDEO_IN_WRITE_PTR_OFFSET             (MEDIA_DEC_REGS_VIDEO_IN_FIFO_REGS_BASE + 0x00)
#define MEDIA_DEC_VIDEO_IN_READ_PTR_OFFSET              (MEDIA_DEC_REGS_VIDEO_IN_FIFO_REGS_BASE + 0x04)
#define MEDIA_DEC_VIDEO_IN_FIFO_SIZE_OFFSET             (MEDIA_DEC_REGS_VIDEO_IN_FIFO_REGS_BASE + 0x08)
#define MEDIA_DEC_VIDEO_IN_FIFO_ENTRY_OFFSET            (MEDIA_DEC_REGS_VIDEO_IN_FIFO_REGS_BASE + 0x0c)
#define MEDIA_DEC_VIDEO_IN_FIFO_START_OFFSET            (MEDIA_DEC_REGS_VIDEO_IN_FIFO_REGS_BASE + 0x10)
#define MEDIA_DEC_VIDEO_IN_FIFO_OFFSET                  (MEDIA_DEC_REGS_VIDEO_IN_FIFO_REGS_BASE + 0x20)

#define MEDIA_DEC_REGS_AUDIO_OUT_FIFO_REGS_BASE         MEDIA_DEC_NEXT(MEDIA_DEC_VIDEO_IN_FIFO_OFFSET, MEDIA_DEC_REGS_VIDEO_FIFO_ENTRIES * sizeof(MEDIA_DEC_FIFO_ENTRY_T), 32)
#define MEDIA_DEC_AUDIO_OUT_WRITE_PTR_OFFSET            (MEDIA_DEC_REGS_AUDIO_OUT_FIFO_REGS_BASE + 0x00)
#define MEDIA_DEC_AUDIO_OUT_READ_PTR_OFFSET             (MEDIA_DEC_REGS_AUDIO_OUT_FIFO_REGS_BASE + 0x04)
#define MEDIA_DEC_AUDIO_OUT_FIFO_SIZE_OFFSET            (MEDIA_DEC_REGS_AUDIO_OUT_FIFO_REGS_BASE + 0x08)
#define MEDIA_DEC_AUDIO_OUT_FIFO_ENTRY_OFFSET           (MEDIA_DEC_REGS_AUDIO_OUT_FIFO_REGS_BASE + 0x0c)
#define MEDIA_DEC_AUDIO_OUT_FIFO_START_OFFSET           (MEDIA_DEC_REGS_AUDIO_OUT_FIFO_REGS_BASE + 0x10)
#define MEDIA_DEC_AUDIO_OUT_FIFO_OFFSET                 (MEDIA_DEC_REGS_AUDIO_OUT_FIFO_REGS_BASE + 0x20)

#define MEDIA_DEC_REGS_AUDIO_IN_FIFO_REGS_BASE          MEDIA_DEC_NEXT(MEDIA_DEC_AUDIO_OUT_FIFO_OFFSET, MEDIA_DEC_REGS_AUDIO_FIFO_ENTRIES * sizeof(MEDIA_DEC_FIFO_ENTRY_T), 32)
#define MEDIA_DEC_AUDIO_IN_WRITE_PTR_OFFSET             (MEDIA_DEC_REGS_AUDIO_IN_FIFO_REGS_BASE + 0x00)
#define MEDIA_DEC_AUDIO_IN_READ_PTR_OFFSET              (MEDIA_DEC_REGS_AUDIO_IN_FIFO_REGS_BASE + 0x04)
#define MEDIA_DEC_AUDIO_IN_FIFO_SIZE_OFFSET             (MEDIA_DEC_REGS_AUDIO_IN_FIFO_REGS_BASE + 0x08)
#define MEDIA_DEC_AUDIO_IN_FIFO_ENTRY_OFFSET            (MEDIA_DEC_REGS_AUDIO_IN_FIFO_REGS_BASE + 0x0c)
#define MEDIA_DEC_AUDIO_IN_FIFO_START_OFFSET            (MEDIA_DEC_REGS_AUDIO_IN_FIFO_REGS_BASE + 0x10)
#define MEDIA_DEC_AUDIO_IN_FIFO_OFFSET                  (MEDIA_DEC_REGS_AUDIO_IN_FIFO_REGS_BASE + 0x20)


#define MEDIA_DEC_FLAGS_OFFSET                          MEDIA_DEC_NEXT(MEDIA_DEC_AUDIO_IN_FIFO_OFFSET, MEDIA_DEC_REGS_AUDIO_FIFO_ENTRIES * sizeof(MEDIA_DEC_FIFO_ENTRY_T), 32)
#define MEDIA_DEC_VOLUME_OFFSET                         (MEDIA_DEC_FLAGS_OFFSET + 4)

// Flags for the FLAGS field above
#define MEDIA_DEC_FLAGS_PAUSED      0x01
#define MEDIA_DEC_FLAGS_MUTED       0x02

#define MEDIA_DEC_TARGET_FULLSCREEN_OFFSET              (MEDIA_DEC_VOLUME_OFFSET + 4)
#define MEDIA_DEC_TARGET_TRANSFORM_OFFSET               (MEDIA_DEC_TARGET_FULLSCREEN_OFFSET + 4)

#define MEDIA_DEC_TRANSPARENCY_OFFSET                   (MEDIA_DEC_TARGET_TRANSFORM_OFFSET + 4)
#define MEDIA_DEC_VIDEO_LEVEL_OFFSET                    (MEDIA_DEC_TRANSPARENCY_OFFSET + 4)
#define MEDIA_DEC_AUDIO_LEVEL_OFFSET                    (MEDIA_DEC_VIDEO_LEVEL_OFFSET + 4)

/* This is a bitfield flag that specifies whether VC has completed a command and
   is acknowledging the ARM kernel when the interrupt is fired. Note: only a single
   bit is needed because, by requirement, there is only ever one thread feeding
   commands into the kernel and thus into VC, and since each command waits on
   acknowledgement before returning, there can never be a second command issued
   (and thus completed) until the last command has been completed and acknowledged.

   Note that this is a waste of 31 bits, but we do it because the kernel ISR will
   be writing into this space as well to clear the ack bit, and we don't want to
   risk any contention/deadlock issues with VC writing to the other bits at
   the same time.
*/
#define MEDIA_DEC_COMMAND_ACK_OFFSET                     (MEDIA_DEC_AUDIO_LEVEL_OFFSET + 4)

/* ---- Callback ring buffer -------------------------------------------------------------------------*/

/* This section is set up so that we can have a buffer of callbacks going back up to the ARM
   kernel, to ensure that multiple coalescing interrupts don't result in lost callbacks.

   We have several components:
*/

/* - An individual callback has two parts: a reason and some data. For space reasons,
     we keep these packed into 72 bit (9 bytes) each, with 8 bits being the reason enum and
     64 bits remaining for data that is appropriate for that reason. Note that reason must come last,
     though, to prevent any padding/alignment issues between VC and the ARM.

     See above for each reason definition.
*/
typedef struct
{
   uint64_t mData;
   uint32_t mReason; /* Should be one of the MDEC_CALLBACK_REASON enums defined in mdec_codecs.h */
   uint32_t mContext;
} MDEC_CALLBACK_INFO;
#define MDEC_CALLBACK_INFO_SIZE_IN_BYTES sizeof(MDEC_CALLBACK_INFO)

/* - The collection of callback reasons are kept in a ring buffer, which means we have
     the array, a head index and a tail index. Head points to where the next callback
     WILL be inserted by VC, tail points to where the next callback should be pulled by the
     ARM kernel. Kindly enough, the initial condition is for all entries to be 0, so head
     and tail will properly be initialized conveniently to be an empty ring at index 0.

     Head and tail are both kept at 32-bits un-modulated. The un-modulated is so we can
     utilize every element in the ring buffer when it's full (otherwise, a "full" condition
     results in a single empty element). The 32-bits is because we're going to rely on
     a single producer/consumer model for the ring buffer between the ARM and VC, which means
     we can get away without any memory protection on the ring buffer, but ONLY if the head
     and tail are atomic read/writes, which means they MUST be 32-bits in size and 32-bit
     aligned.

     Also, make sure the size is a multiple of 8. That way, it's a power of 2, so the modulo
     can be done with bitmasks (defined below), AND the total size of the ring buffer ends
     up being 32-bit aligned, so the remaining register space won't end up misaligned.
*/
#define MEDIA_DEC_REGS_CALLBACK_FIFO_REGS_BASE           (MEDIA_DEC_COMMAND_ACK_OFFSET + 4)
#define MEDIA_DEC_CALLBACKS_HEAD_OFFSET                  (MEDIA_DEC_REGS_CALLBACK_FIFO_REGS_BASE + 0)
#define MEDIA_DEC_CALLBACKS_TAIL_OFFSET                  (MEDIA_DEC_REGS_CALLBACK_FIFO_REGS_BASE + 4)

#define MEDIA_DEC_CALLBACK_BUFFER_SIZE                   (1 << 6) // Must be a power of 2
#define MEDIA_DEC_CALLBACK_BUFFER_SIZE_MASK              (MEDIA_DEC_CALLBACK_BUFFER_SIZE - 1)

#define MEDIA_DEC_CALLBACKS_RING_BUFFER_OFFSET           MEDIA_DEC_NEXT(MEDIA_DEC_REGS_CALLBACK_FIFO_REGS_BASE, 8, 32)
#define MEDIA_DEC_CALLBACKS_RING_BUFFER_END_OFFSET       (MEDIA_DEC_CALLBACKS_RING_BUFFER_OFFSET + MEDIA_DEC_CALLBACK_BUFFER_SIZE * MDEC_CALLBACK_INFO_SIZE_IN_BYTES )


/* ---- Setup config area ----------------------------------------------------------------------*/
/* This area is used to store the config options passed in to setup() for audio and video
   configuration parameters.
*/

/* Audio config area. The data block here is filled with one of the MDEC_*_CONFIG structs */
/* Max size of any of those structs is 5*4=20 bytes */
#define MEDIA_DEC_AUDIO_CONFIG_OFFSET                   MEDIA_DEC_NEXT(MEDIA_DEC_CALLBACKS_RING_BUFFER_END_OFFSET, 0, 32)

#define MEDIA_DEC_REGS_MAX_AUDIO_CONFIG_BYTES           0x20
#define MEDIA_DEC_REGS_SIZE                             (MEDIA_DEC_AUDIO_CONFIG_OFFSET + MEDIA_DEC_REGS_MAX_AUDIO_CONFIG_BYTES)


#endif // MEDIA_DEC_REGS_H_
