#include <common.h>
#include <command.h>
#include <asm/arch/vcio.h>
#include <asm/io.h>
#include <asm/arch/ipc.h>


/*******************************************************************/
/* bootvc - boot vc image from image in memory */
/*******************************************************************/
#define VC_BOOTARGS_START   3 //First argv that needs to be passed to VC
#define MAX_ALLOWED_OFFSET  0x800 //Maximum space available to write arguments (2k)

// Don't allow multiple boots.
static int vc_init = 0;

int do_bootvc (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	ulong	addr;
	int     rc = 0;

	if (argc < 2) {
		cmd_usage(cmdtp);
		return rc;
	}

	if (vc_init == 0) {
		addr = simple_strtoul(argv[1], NULL, 16);
		printf("Booting videcore at 0x%08lx vc_phy_addr: 0x%08lx\n", addr, __ipc_phys_to_bus(addr));

		//Store the boot args if present
		if(argc > VC_BOOTARGS_START){
			uint32_t *args_base_addr = (uint32_t *) (simple_strtoul(argv[2], NULL, 16));
			uint32_t *args_addr = args_base_addr;
			uint32_t vc_argc = argc - VC_BOOTARGS_START; //Total number of arguments that are passed to VC
			uint32_t running_offset = 0, i=0;

			printf("vcboot: found vc bootargs location at %p with %d boot args\n", args_addr, vc_argc);

			//Store the total argument count for VC at the base address
			*args_addr++ = vc_argc;

			//Initialize the running_offset with the first offset:
			//The first offset is calculated as one word of each argument plus one word to store the argument count
			running_offset = (vc_argc + 1) * sizeof(vc_argc);

			//Now iterate over argument count to save offset and string for each argument
			for(i=0; i< vc_argc; i++){
				*args_addr++ = running_offset;

				//Check to ensure that this strcpy will not go beyond max available space for arguments
				if( running_offset + strlen( argv[i + VC_BOOTARGS_START] ) >= MAX_ALLOWED_OFFSET ){
					printf("vcboot: Not enough space to add another vc_bootargs \n\tTruncating vc_bootargs \n");
					*args_base_addr = i; //Fixup the argument count refelect number of actual arguments written
					break;
				}

				//Copy the argument at correct location
				strcpy( (char *)( (char *)args_base_addr + running_offset), argv[i + VC_BOOTARGS_START] );

				//Update the running offset
				running_offset += strlen( argv[i + VC_BOOTARGS_START] ) + 1; //Add an extra byte for NULL termination
			}
		}

		bcm_mailbox_write(0, __ipc_phys_to_bus(addr));

		//Reinitialize the serial port registers as may clear out these registers
		serial_init();

		rc = ipc_init();
		vc_init++;
	} else {
		printf("VC init already called, skipping...\n");
	}

	return rc;
}



U_BOOT_CMD(
		bootvc, CONFIG_SYS_MAXARGS, 1, do_bootvc,
		"boot videocore with appplication image at address 'vc_bootaddr'",
		"bootvc 0x0d000000 0x0c002800 reloc_size=33554432 dmalloc_size=48371936 \n"
		"[vc_bootaddr] - boot videocore with application image at address 'addr' (in hexadecimal)\n"
		"[vc_argsaddr] - optional: store VC boot arguments at vc_argsaddr (in hexadecimal) \n"
		"[vc_arg1]     - optional: first argument 'reloc_size=33554432' (reloc size in bytes)\n"
		"[vc_arg2]     - optional: second argument 'dmalloc_size=48371936' (dmalloc size in bytes)\n"
		);


