#include <malloc.h>
#include <common.h>
#include <command.h>
#include <asm/arch/vcio.h>
#include <asm/arch/ipc.h>
#include <asm/io.h>
#include "otp_regs.h"
#include "otp.h"

/*******************************************************************/
/* otpvc - program otp                                             */
/*******************************************************************/
int do_otpvc (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    uint32_t addr, length;
    int      rc = 0;

    // Check otp IPC.
    if(otp_init()) {
        return -1;
    }

    if (argc < 4) {
        cmd_usage(cmdtp);
        return rc;
    } else if (argc >= 4) {
        uint32_t *data = NULL;

        addr = simple_strtoul(argv[2], NULL, 16);
        length = simple_strtoul(argv[3], NULL, 16);
        if (argc >=5 ) {
            data = (uint32_t *)simple_strtoul(argv[4], NULL, 16);
        }

        if (0 == strncmp(argv[1], "read", 4)) {
            uint32_t *otp_data = NULL;

            printf("setup read from 0x%08x with length 0x%08x\n", addr, length);

            otp_data = otp_read(addr, length, 0);
            // convert to hex
            if (otp_data) {
                int i = 0;
                printf("Read:\n");
                for (i = 0; i < length; i++) {
                    printf("%d 0x%08x\n", i+addr, otp_data[i]);
                }
                if (argc >= 5) {
                    memcpy(data, otp_data, (length * 4));
                }
            }
        } else if (0 == strncmp(argv[1], "write",5) && argc >= 5) {
            printf("writing %d rows(4 bytes) from 0x%08x to otp addr 0x%08x\n", length, (uint32_t)data, addr);
            rc = otp_write(addr, (uint32_t *)data, length);
            if (rc != 0 ) {
                printf("write failed\n");
            }
        } else {
            cmd_usage(cmdtp);
        }
        return rc;
    }

    return rc;
}


U_BOOT_CMD(
        otpvc, CONFIG_SYS_MAXARGS, 0, do_otpvc,
        "program otp with data at address 'addr'. length is in rows",
        "read <addr> <length> <buf> | write <addr> <length> <data>\n"
        "          arguments in hexadecimal"
        );

