#include <common.h>
#include <asm/arch/vcio.h>
#include <asm/arch/ipc.h>
#include <asm/io.h>

#include <asm/arch/media_dec_regs.h>
#include "otp.h"
#include "otp_regs.h"

/*******************************************************************/
/* otpvc - program otp                                             */
/*******************************************************************/
static uint32_t otp_id = 0;
static uint32_t otp_base = 0;

int otp_init(void) {
    // Check VC Media Dec IPC.
    char fourcc[4] = "OTPP";
    if (otp_base == 0) {
        otp_base = ipc_get_service_addr(*(uint32_t *)fourcc, &otp_id);
        printf("Starting otp, base = %x id=%d\n", otp_base, otp_id);
        if (!otp_base) {
            printf("Error: Failed to get otp base addr\n");
            return -1;
        }
    }
    return 0;
}

void otp_shutdown(void) {
    otp_id = 0;
    otp_base = 0;
}

// Read length rows from otp addr. If ignore_crc, will still return data ptr
// on errors (not safe!).
uint32_t* otp_read(uint32_t addr, uint32_t length, uint32_t ignore_crc)
{
    volatile uint32_t val;
    uint32_t crc;
    uint32_t *data;

    if (((length * 4) > OTP_REQUEST_BUFFER_SIZE) || (length>OTP_MAX_ROWS)) {
        printf("otp_read: Invalid parameter\n");
        return NULL;
    }

    writel(addr, otp_base + OTP_ADDRESS_OFFSET);
    writel(length, otp_base + OTP_REQUEST_LENGTH_OFFSET);
    writel(OTP_CONTROL_GO | OTP_CONTROL_READ, otp_base + OTP_CONTROL_OFFSET);
    ipc_notify_vc_event(otp_id);

    val = readl(otp_base + OTP_CONTROL_OFFSET);
    while(!(val & OTP_CONTROL_DONE)) {
        val = readl(otp_base + OTP_CONTROL_OFFSET);
    }

    if (readl(otp_base + OTP_STATUS_OFFSET)) {
        printf("otp_read: status error 0x%08x\n", readl(otp_base + OTP_STATUS_OFFSET));
        return NULL;
    }

    data = (uint32_t*)(otp_base + OTP_REQUEST_DATA_OFFSET);
    // length is in OTP row length(4 bytes)
    crc = crc32(0, (unsigned char *)data, length * 4);

    if (crc != readl(otp_base + OTP_DATA_CRC_OFFSET)) {
        printf("otp_read: data crc mismatch read %08x != %08x\n",
                crc, readl(otp_base + OTP_DATA_CRC_OFFSET));
        if (!ignore_crc) {
            return NULL;
        }
    } else {
        printf("CRC = %08x\n", crc);
    }

    return data;
}

int otp_write(uint32_t addr, uint32_t* data, uint32_t length)
{
    volatile uint32_t val;
    uint32_t crc;

    if (((length * 4) > OTP_REQUEST_BUFFER_SIZE) || (length>OTP_MAX_ROWS)) {
        printf("otp_write: Invalid parameter\n");
        return -1;
    }

    writel(addr, otp_base + OTP_ADDRESS_OFFSET);
    writel(length, otp_base + OTP_REQUEST_LENGTH_OFFSET);
    memcpy((uint32_t *)(otp_base + OTP_REQUEST_DATA_OFFSET), data, length);
    // Length is in OTP row length (4 bytes)
    crc = crc32(0, (unsigned char *)data, length * 4);
    writel(crc, otp_base + OTP_DATA_CRC_OFFSET);
    writel(OTP_CONTROL_GO, otp_base + OTP_CONTROL_OFFSET);
    ipc_notify_vc_event(otp_id);

    val = readl(otp_base + OTP_CONTROL_OFFSET);
    while(!(val & OTP_CONTROL_DONE)) {
        val = readl(otp_base + OTP_CONTROL_OFFSET);
    }

    if (readl(otp_base + OTP_STATUS_OFFSET)) {
        printf("otp_write: status error\n");
        return -1;
    }

    return 0;
}

