#ifndef TVIF_H_
#define TVIF_H_
#include "tv_interface_regs.h"

// CEA COPIED FROM interface/vmcs_host/vc_hdmi.h

//These are CEA mode numbers (sent in AVI infoframe) for different resolutions
//1080i at 100/120Hz (40,46) are supported by HDMI H/W but note we cannot
//display the debug overlay under these modes
typedef enum {
   HDMI_CEA_VGA             =  1,
   HDMI_CEA_480p60          =  2,
   HDMI_CEA_480p60H         =  3,
   HDMI_CEA_720p60          =  4,
   HDMI_CEA_1080i60         =  5,
   HDMI_CEA_480i60          =  6,
   HDMI_CEA_480i60H         =  7,
   HDMI_CEA_240p60          =  8,
   HDMI_CEA_240p60H         =  9,
   HDMI_CEA_480i60_4x       = 10,
   HDMI_CEA_480i60_4xH      = 11,
   HDMI_CEA_240p60_4x       = 12,
   HDMI_CEA_240p60_4xH      = 13,
   HDMI_CEA_480p60_2x       = 14,
   HDMI_CEA_480p60_2xH      = 15,
   HDMI_CEA_1080p60         = 16,
   HDMI_CEA_576p50          = 17,
   HDMI_CEA_576p50H         = 18,
   HDMI_CEA_720p50          = 19,
   HDMI_CEA_1080i50         = 20,
   HDMI_CEA_576i50          = 21,
   HDMI_CEA_576i50H         = 22,
   HDMI_CEA_288p50          = 23,
   HDMI_CEA_288p50H         = 24,
   HDMI_CEA_576i50_4x       = 25,
   HDMI_CEA_576i50_4xH      = 26,
   HDMI_CEA_288p50_4x       = 27,
   HDMI_CEA_288p50_4xH      = 28,
   HDMI_CEA_576p50_2x       = 29,
   HDMI_CEA_576p50_2xH      = 30,
   HDMI_CEA_1080p50         = 31,
   HDMI_CEA_1080p24         = 32,
   HDMI_CEA_1080p25         = 33,
   HDMI_CEA_1080p30         = 34,
   HDMI_CEA_480p60_4x       = 35,
   HDMI_CEA_480p60_4xH      = 36,
   HDMI_CEA_576p50_4x       = 37,
   HDMI_CEA_576p50_4xH      = 38,
   HDMI_CEA_1080i50_rb      = 39,
   HDMI_CEA_1080i100        = 40,
   HDMI_CEA_720p100         = 41,
   HDMI_CEA_576p100         = 42,
   HDMI_CEA_576p100H        = 43,
   HDMI_CEA_576i100         = 44,
   HDMI_CEA_576i100H        = 45,
   HDMI_CEA_1080i120        = 46,
   HDMI_CEA_720p120         = 47,
   HDMI_CEA_480p120         = 48,
   HDMI_CEA_480p120H        = 49,
   HDMI_CEA_480i120         = 50,
   HDMI_CEA_480i120H        = 51,
   HDMI_CEA_576p200         = 52,
   HDMI_CEA_576p200H        = 53,
   HDMI_CEA_576i200         = 54,
   HDMI_CEA_576i200H        = 55,
   HDMI_CEA_480p240         = 56,
   HDMI_CEA_480p240H        = 57,
   HDMI_CEA_480i240         = 58,
   HDMI_CEA_480i240H        = 59,
   //Put more CEA codes here if we support in the future

   HDMI_CEA_OFF = 0xff //Special code to shutdown HDMI

} HDMI_CEA_RES_CODE_T;


//Two main types of displays (CEA - HDMI device, DMT - computer monitors)
typedef enum {
   HDMI_RES_GROUP_INVALID = 0,
   HDMI_RES_GROUP_CEA,
   HDMI_RES_GROUP_DMT,
   HDMI_RES_GROUP_CUSTOM //Usused at the moment
} HDMI_RES_GROUP_T;

/**
 * TV output control
 */
typedef enum {
   TV_INTF_CTRL_INVALID = 0,      ///< Disallow cleared memory
   TV_INTF_CTRL_OFF,              ///< No output at all (device appears to be powered down)
   TV_INTF_CTRL_HDMI,             ///< Output to HDMI
   TV_INTF_CTRL_SDTV,             ///< Output to composite
   TV_INTF_CTRL_AUTO,             ///< Output to HDMI if plugged in otherwise composite
} TV_INTF_CTRL_T;

int tvif_init(void);
void tvif_shutdown(void);

int tvif_set_output(TV_INTF_CTRL_T mode);
int tvif_set_hdmi_resolution(HDMI_CEA_RES_CODE_T mode);

#endif
